// package.h   -*- C++ -*-
// Ian Lance Taylor for ICFP 2002

// Information about packages.

#ifndef PACKAGE_H
#define PACKAGE_H

#include <set>
#include <vector>
#include <map>
#include <iostream>

#include "socket.h"

class Board;
class Player;
class Robots;

typedef std::pair<short, short> Xy;

typedef std::set<int> Package_list;

class Package
{
public:
  Package()
    : m_x(-1), m_y(-1), m_robot(-1), m_dest_x(-1), m_dest_y(-1), m_weight(-1)
  { }

  bool xy_is_known() const
  { return this->m_x != -1; }
  int x() const
  { return this->m_x; }
  int y() const
  { return this->m_y; }
  Xy xy() const
  { return Xy(this->m_x, this->m_y); }
  int weight() const
  { return this->m_weight; }

  bool dest(int& x, int& y) const;
  Xy dest() const
  { return Xy(this->m_dest_x, this->m_dest_y); }

  void at(int x, int y);
  void with(int robot);
  void info(int x, int y, int dest_x, int dest_y, int weight);

private:
  int m_x;
  int m_y;
  int m_robot;
  int m_dest_x;
  int m_dest_y;
  int m_weight;
};

class Packages
{
public:
  Packages()
    : m_packages(), m_package_locs()
  { }

  Package_list read(const Player&, const Robots&, Socket&, int x, int y);

  // Return the weight of a package specified by ID.  -1 if not known.
  int weight(int pid) const;

  // Return the destination of a package specified by ID.  Returns
  // false if not known.
  bool dest(int pid, int& x, int& y) const;
  Xy dest(int pid) const;

  // A package is known to be at (x, y).
  void at(int pid, int x, int y);

  // A package was picked up by robot rid.
  void with(int pid, int rid);

  // Return the number of packages at (x, y).
  size_t count(int x, int y) const;

  // Return the location nearest to (x, y) which has at least one
  // package.  Return false if none known.
  bool nearest_package(Board&, int x, int y, int& xn, int& yn) const;

private:
  typedef std::vector<Package> Package_vector;
  typedef std::map<Xy, Package_list> Package_locs;

  Package_vector m_packages;
  Package_locs m_package_locs;
};

extern std::ostream& operator<<(std::ostream&, const Package_list&);

#endif // ! defined(PACKAGE_H)
