// player.h  -*- C++ -*-
// Ian Lance Taylor for ICFP 2002

// Hold information for the player.

#ifndef PLAYER_H
#define PLAYER_H

#include "socket.h"
#include "robots.h"

class Player
{
public:
  Player()
    : m_id(0), m_carry(0), m_money(0), m_bumped(0)
  { }

  void read(Socket&);

  int id() const
  { return this->m_id; }
  int carry() const
  { return this->m_carry; }
  int money() const
  { return this->m_money; }

  void location(const Robots& robots, int& x, int& y) const
  { robots.location(this->m_id, x, y); }
  Xy xy(const Robots& robots) const
  { return robots.xy(this->m_id); }

  int score(const Robots& robots) const
  { return robots.score(this->m_id); }

  void pay(int bid)
  { this->m_money -= bid; }

  Package_list packages(const Robots& robots) const
  { return robots.packages(this->m_id); }

  // Return the total weight of packages we are carrying.
  int weight(const Robots& robots, const Packages& packs) const
  { return robots.weight(this->m_id, packs); }

  // Record the number of times we were bumped in the last ten turns.
  void bumped(bool yes);
  int bumped_count();

  friend std::ostream& operator<<(std::ostream&, const Player&);

private:
  int m_id;
  long m_carry;
  long m_money;
  unsigned int m_bumped;
};

#endif // ! defined(PLAYER_H)
