// robots.h -*- C++ -*-
// Ian Lance Taylor for ICFP 2002

// Keep track of all robots in the game.

#ifndef ROBOTS_H
#define ROBOTS_H

#include <iostream>
#include <vector>
#include <list>

#include "socket.h"
#include "package.h"
#include "board.h"

class Player;

class Robot
{
public:
  Robot()
    : m_turn(-1), m_x(0), m_y(0), m_packages(), m_score(0)
  { }

  Robot(int x, int y)
    : m_turn(Robot::turn), m_x(x), m_y(y), m_packages()
  { }

  bool was_valid() const
  { return this->m_turn == Robot::turn - 1; }
  bool is_valid() const
  { return this->m_turn == Robot::turn; }
  void mark_valid()
  { this->m_turn = Robot::turn; }

  static void change_turn()
  { ++Robot::turn; }

  int x() const
  { return this->m_x; }
  int y() const
  { return this->m_y; }
  void location(int& x, int& y) const;
  Xy xy() const
  { return Xy(this->m_x, this->m_y); }

  int score() const
  { return this->m_score; }

  void set(int x, int y);
  void move(Direction);
  void pickup(int);
  void drop(int);

  Package_list packages() const
  { return this->m_packages; }

  // Return total weight of packages robot is known to be carrying.
  int weight(const Packages&) const;

  void scored(int add)
  { this->m_score += add; }

  friend std::ostream& operator<<(std::ostream&, const Robot&);

private:
  static int turn;

  int m_turn;
  int m_x;
  int m_y;
  Package_list m_packages;
  int m_score;
};

class Robots
{
public:
  Robots()
  { }

  void read(const Player&, Socket&, Packages&);

  void location(int id, int& x, int& y) const;
  Xy xy(int id) const;

  int score(int id) const;

  Package_list packages(int id) const;

  int weight(int id, const Packages&) const;

  friend std::ostream& operator<<(std::ostream&, const Robots&);

  // Return the ID of the robot at (x, y), or -1 if there is none.
  // (It is possisble to have more than one robot at the same location
  // in weird situations, but we ignore that.)
  int at(int x, int y) const;

private:
  std::vector<Robot> m_robots;
};

#endif // ! defined(ROBOTS_H)
