// socket.h  -*- C++ -*-
// Ian Lance Taylor for ICFP 2002

// Simple socket class to handle I/O.

#ifndef SOCKET_H
#define SOCKET_H

#include <string>
#include <list>

class Buffers
{
public:
  Buffers()
    : m_buffers()
  { }

  ~Buffers()
  { this->clear(); }

  static const size_t page_size;

  // Amount of data in buffer.
  size_t size() const;

  // Add data to a buffer.
  void add_buff(char* buff, size_t len);
  void add(const void* buff, size_t len);
  void read(int o);

  // Consume data from a buffer.
  size_t consume(size_t len);

  // Find a character.
  bool find(unsigned char c, size_t& loc) const;

  // Move bytes out of a buffer.
  size_t move(void* buff, size_t len);
  size_t move_string(std::string& s, size_t len);

private:

  void clear();

  struct Buffer
  {
    char* alc;
    char* data;
    size_t len;
  };

  typedef std::list<Buffer> Buffer_list;

  Buffer_list m_buffers;
};

class Socket
{
public:
  Socket(int o)
    : m_o(o), m_buffers()
  { }

  void write(const void* buff, size_t len);
  void write(const char*);
  void write(const std::string&);

  void writeln(const char* s);
  void writeln(const std::string&);

  void read(void* buff, size_t len);
  std::string getline();

private:
  int m_o;
  Buffers m_buffers;
};

#endif // ! defined(SOCKET_H)
