// state.c
// Ian Lance Taylor for ICFP 2002

// Implementation of State.

#include "package.h"
#include "board.h"
#include "player.h"
#include "robots.h"
#include "state.h"

State::State(
  Board& board,
  Player& me,
  Robots& robots,
  Packages& packs)
  : m_board(board), m_me(me), m_robots(robots), m_packs(packs),
    m_found_robots(false), m_nearby_robots()
{
  this->m_me.location(this->m_robots, this->m_xme, this->m_yme);
}

class Find_robots : public Board::Find
{
public:
  Find_robots(Robots& robots, State::Robot_list& set)
    : m_robots(robots), m_set(set)
  { }
  bool operator()(Board&, int x, int y, int distance);
private:
  Robots& m_robots;
  State::Robot_list& m_set;
};

bool
Find_robots::operator()(
  Board&,
  int x,
  int y,
  int distance)
{
  // Ignore myself.
  if (distance == 0)
    { return false; }
  int rid = this->m_robots.at(x, y);
  if (rid != -1)
    { this->m_set.push_back(State::Nearby_robot(rid, x, y, distance)); }
  // Always return false so we keep looking.
  return false;
}

State::Robot_list
State::nearby_robots()
{
  if (! this->m_found_robots)
    {
      Find_robots find_robots(this->m_robots, this->m_nearby_robots);
      int xret;
      int yret;
      int distanceret;
      this->m_board.find(this->m_xme, this->m_yme, find_robots, 5,
			 xret, yret, distanceret);
      this->m_found_robots = true;
    }
  return this->m_nearby_robots;
}
