// state.h   -*- C++ -*-
// Ian Lance Taylor for ICFP 2002

// State information for the various strategies.

#ifndef STATE_H
#define STATE_H

#include "package.h"
#include "board.h"
#include "player.h"
#include "robots.h"

class State
{
public:
  State(Board&, Player&, Robots&, Packages&);

  Board& board()
  { return this->m_board; }
  Player& me()
  { return this->m_me; }
  Robots& robots()
  { return this->m_robots; }
  Packages& packs()
  { return this->m_packs; }

  int xme() const
  { return this->m_xme; }
  int yme() const
  { return this->m_yme; }
  Xy xyme() const
  { return Xy(this->m_xme, this->m_yme); }

  // Return a list of other robots within five moves of the current
  // position.
  struct Nearby_robot
  {
    int id;
    int x;
    int y;
    int distance;

    Nearby_robot(int ida, int xa, int ya, int distancea)
      : id(ida), x(xa), y(ya), distance(distancea)
    { }
  };

  typedef std::list<Nearby_robot> Robot_list;
  Robot_list nearby_robots();

private:
  Board& m_board;
  Player& m_me;
  Robots& m_robots;
  Packages& m_packs;

  int m_xme;
  int m_yme;

  bool m_found_robots;
  Robot_list m_nearby_robots;
};

#endif // ! defined(STATE_H)
